//+------------------------------------------------------------------+
//|                                                      ProjectName |
//|                                      Copyright 2011, CompanyName |
//|                                       http://www.companyname.net |
//+------------------------------------------------------------------+
/*
*     SmoothAlgorithms.mqh  
*  : __\MQL5\Include
*/
//+------------------------------------------------------------------+
//|                                                         XCHO.mq5 | 
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link "farria@mail.redcom.ru"
#property description "Chaikin Oscillator"
//----   
#property version   "1.00"
//----     
#property indicator_separate_window
//----   
#property indicator_buffers 1 
//----     
#property indicator_plots   1
//+-----------------------------------+
//|       |
//+-----------------------------------+
//----     
#property indicator_type1   DRAW_LINE
//----        Orange
#property indicator_color1 Orange
//----   -  
#property indicator_style1  STYLE_SOLID
//----     1
#property indicator_width1  1
//----    
#property indicator_label1  "XCHO"

//+-----------------------------------+
//|    CXMA             |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+

//----    CXMA   SmoothAlgorithms.mqh
CXMA XMA1,XMA2;
//+-----------------------------------+
//|             |
//+-----------------------------------+
/*enum Smooth_Method -    SmoothAlgorithms.mqh
  {
   MODE_SMA_,  // SMA
   MODE_EMA_,  // EMA
   MODE_SMMA_, // SMMA
   MODE_LWMA_, // LWMA
   MODE_JJMA,  // JJMA
   MODE_JurX,  // JurX
   MODE_ParMA, // ParMA
   MODE_T3,    // T3
   MODE_VIDYA, // VIDYA
   MODE_AMA,   // AMA
  }; */
//+-----------------------------------+
//|         |
//+-----------------------------------+
input int FastMA=3;                //   
input int SlowMA=10;               //   
input int Phase_=15;               //  ,
                                   //  JJMA    -100 ... +100,     ;
//  VIDIA   CMO,  AMA    
input Smooth_Method SmoothMethod=MODE_T3;          //  
input ENUM_APPLIED_VOLUME VolumeType=VOLUME_TICK;  // 
input int Shift=0;                                 //      
//+-----------------------------------+

//----   ,   
//----      
double XCHO[];

//----      
int StartBars;
//+------------------------------------------------------------------+
//| calculate AD                                                     |
//+------------------------------------------------------------------+
double GetAD(const double &high[],
             const double &low[],
             const double &close[],
             const long &tick_volume[],
             const long &volume[],
             uint bar)

  {
   long vol;
   double res=0;
//---
   if(VolumeType==VOLUME_TICK)
      vol=tick_volume[bar];
   else vol=volume[bar];

   if(high[bar]!=low[bar])
      res=(2*close[bar]-high[bar]-low[bar])/(high[bar]-low[bar])*vol;
//---
   return(res);
  }
//+------------------------------------------------------------------+   
//| XCHO indicator initialization function                           | 
//+------------------------------------------------------------------+ 
void OnInit()
  {
//----     
   StartBars=MathMax(XMA1.GetStartBars(SmoothMethod,FastMA,Phase_),
                     XMA2.GetStartBars(SmoothMethod,SlowMA,Phase_));

//----       
   XMA1.XMALengthCheck("FastMA", FastMA);
   XMA2.XMALengthCheck("SlowMA", SlowMA);
//----       
   XMA1.XMAPhaseCheck("Phase_",Phase_,SmoothMethod);

//----      
   SetIndexBuffer(0,XCHO,INDICATOR_DATA);
//----    1  
   PlotIndexSetInteger(0,PLOT_SHIFT,Shift);
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,StartBars);
//---      DataWindow
   PlotIndexSetString(0,PLOT_LABEL,"XCHO");
//----   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//----      
   string shortname,Smooth=XMA1.GetString_MA_Method(SmoothMethod);
   StringConcatenate(shortname,"Chaikin Oscillator(",string(SlowMA),",",string(FastMA),",",Smooth,")");
//----           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);

//---     
   IndicatorSetInteger(INDICATOR_DIGITS,0);
//----  
  }
//+------------------------------------------------------------------+ 
//| XCHO iteration function                                          | 
//+------------------------------------------------------------------+ 
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(rates_total<StartBars) return(0);

//----       
   double ad=0,slowma,fastma;
//----   
   int first,bar;
//----     
   static double ad_;

//----    first    
   if(prev_calculated>rates_total || prev_calculated<=0) //      
     {
      first=0; //      
      ad_=GetAD(high,low,close,tick_volume,volume,0);
     }
   else first=prev_calculated-1; //      

//----   
   ad=ad_;

//----    
   for(bar=first; bar<rates_total && !IsStopped(); bar++)
     {
      //----        
      if(rates_total!=prev_calculated && bar==rates_total-1)
        {
         ad_=ad;
        }

      //----    Accumulation Distribution
      ad+=GetAD(high,low,close,tick_volume,volume,bar);

      //----    XMASeries  
      fastma = XMA1.XMASeries(0, prev_calculated, rates_total, SmoothMethod, Phase_, FastMA, ad, bar, false);
      slowma = XMA2.XMASeries(0, prev_calculated, rates_total, SmoothMethod, Phase_, SlowMA, ad, bar, false);

      //----   
      if(bar<StartBars)
        {
         XCHO[bar]=EMPTY_VALUE;
         continue;
        }
      XCHO[bar]=fastma-slowma;
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
